# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/user_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_UserPreferencesPageWidget(object):
    def setupUi(self, UserPreferencesPageWidget):
        UserPreferencesPageWidget.setObjectName("UserPreferencesPageWidget")
        UserPreferencesPageWidget.resize(472, 370)
        self.verticalLayout = QtWidgets.QVBoxLayout(UserPreferencesPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiSettingsGroupBox = QtWidgets.QGroupBox(UserPreferencesPageWidget)
        self.uiSettingsGroupBox.setObjectName("uiSettingsGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiSettingsGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiEmailLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiEmailLabel.setObjectName("uiEmailLabel")
        self.gridLayout.addWidget(self.uiEmailLabel, 3, 0, 1, 1)
        self.uiFullNameLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiFullNameLabel.setObjectName("uiFullNameLabel")
        self.gridLayout.addWidget(self.uiFullNameLabel, 2, 0, 1, 1)
        self.uiUsernameLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiUsernameLabel.setObjectName("uiUsernameLabel")
        self.gridLayout.addWidget(self.uiUsernameLabel, 0, 0, 1, 1)
        self.uiUsernameLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiUsernameLineEdit.setReadOnly(True)
        self.uiUsernameLineEdit.setObjectName("uiUsernameLineEdit")
        self.gridLayout.addWidget(self.uiUsernameLineEdit, 0, 1, 1, 1)
        self.uiFullNameLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiFullNameLineEdit.setReadOnly(True)
        self.uiFullNameLineEdit.setObjectName("uiFullNameLineEdit")
        self.gridLayout.addWidget(self.uiFullNameLineEdit, 2, 1, 1, 1)
        self.uiEmailLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiEmailLineEdit.setReadOnly(True)
        self.uiEmailLineEdit.setObjectName("uiEmailLineEdit")
        self.gridLayout.addWidget(self.uiEmailLineEdit, 3, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiSettingsGroupBox)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.uiChangePasswordPushButton = QtWidgets.QPushButton(UserPreferencesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiChangePasswordPushButton.sizePolicy().hasHeightForWidth())
        self.uiChangePasswordPushButton.setSizePolicy(sizePolicy)
        self.uiChangePasswordPushButton.setObjectName("uiChangePasswordPushButton")
        self.horizontalLayout.addWidget(self.uiChangePasswordPushButton)
        self.uiCopyAccessTokenPushButton = QtWidgets.QPushButton(UserPreferencesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiCopyAccessTokenPushButton.sizePolicy().hasHeightForWidth())
        self.uiCopyAccessTokenPushButton.setSizePolicy(sizePolicy)
        self.uiCopyAccessTokenPushButton.setObjectName("uiCopyAccessTokenPushButton")
        self.horizontalLayout.addWidget(self.uiCopyAccessTokenPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        spacerItem1 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)

        self.retranslateUi(UserPreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(UserPreferencesPageWidget)

    def retranslateUi(self, UserPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        UserPreferencesPageWidget.setWindowTitle(_translate("UserPreferencesPageWidget", "User"))
        self.uiSettingsGroupBox.setTitle(_translate("UserPreferencesPageWidget", "Logged-in user information"))
        self.uiEmailLabel.setText(_translate("UserPreferencesPageWidget", "Email address:"))
        self.uiFullNameLabel.setText(_translate("UserPreferencesPageWidget", "Full name:"))
        self.uiUsernameLabel.setText(_translate("UserPreferencesPageWidget", "Username:"))
        self.uiChangePasswordPushButton.setText(_translate("UserPreferencesPageWidget", "&Change password"))
        self.uiCopyAccessTokenPushButton.setText(_translate("UserPreferencesPageWidget", "&Copy access token"))
