# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/password_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PasswordDialog(object):
    def setupUi(self, PasswordDialog):
        PasswordDialog.setObjectName("PasswordDialog")
        PasswordDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        PasswordDialog.resize(377, 111)
        PasswordDialog.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(PasswordDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.uiConfirmPasswordLineEdit = QtWidgets.QLineEdit(PasswordDialog)
        self.uiConfirmPasswordLineEdit.setMaxLength(100)
        self.uiConfirmPasswordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.uiConfirmPasswordLineEdit.setObjectName("uiConfirmPasswordLineEdit")
        self.gridLayout.addWidget(self.uiConfirmPasswordLineEdit, 1, 1, 1, 1)
        self.uiPasswordLabel = QtWidgets.QLabel(PasswordDialog)
        self.uiPasswordLabel.setObjectName("uiPasswordLabel")
        self.gridLayout.addWidget(self.uiPasswordLabel, 0, 0, 1, 1)
        self.uiPasswordLineEdit = QtWidgets.QLineEdit(PasswordDialog)
        self.uiPasswordLineEdit.setMaxLength(100)
        self.uiPasswordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.uiPasswordLineEdit.setObjectName("uiPasswordLineEdit")
        self.gridLayout.addWidget(self.uiPasswordLineEdit, 0, 1, 1, 1)
        self.uiConfirmPasswordLabel = QtWidgets.QLabel(PasswordDialog)
        self.uiConfirmPasswordLabel.setObjectName("uiConfirmPasswordLabel")
        self.gridLayout.addWidget(self.uiConfirmPasswordLabel, 1, 0, 1, 1)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(PasswordDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.gridLayout.addWidget(self.uiButtonBox, 2, 0, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 3, 0, 1, 1)

        self.retranslateUi(PasswordDialog)
        self.uiButtonBox.accepted.connect(PasswordDialog.accept) # type: ignore
        self.uiButtonBox.rejected.connect(PasswordDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(PasswordDialog)
        PasswordDialog.setTabOrder(self.uiPasswordLineEdit, self.uiConfirmPasswordLineEdit)

    def retranslateUi(self, PasswordDialog):
        _translate = QtCore.QCoreApplication.translate
        PasswordDialog.setWindowTitle(_translate("PasswordDialog", "Change password"))
        self.uiPasswordLabel.setText(_translate("PasswordDialog", "New password:"))
        self.uiConfirmPasswordLabel.setText(_translate("PasswordDialog", "Confirm password:"))
